/*
 * PCI Express Half-Mini Card to Full-Mini-only socket bracket, pin variant
 *
 * Copyright (C) 2019-2020  Libiquity LLC
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/* "Spec" in variable comments refers to PCI Express Mini Card Electromechanical
 * Specification, Revision 1.2. */
card_w = 30.00;  /* Card width (Spec Figures 2-3 and 2-4) */
span_l = 24.15;  /* Half-Mini to Full-Mini length difference (Spec Figures 2-3
                  * and 2-4): 50.95 - 26.80 or 48.05 - 23.90 */
brdg_w =  8.00;  /* Width (parallel to bracket length) of bridge between
                  * component and routing keep out areas; I/O connector area
                  * width (Spec Figure 2-10) works well here */
brdg_f =  2.00;  /* Bridge to body fillet radius */
hole_s = 24.20;  /* Hold down solution hole spacing (Spec Figure 2-3) */
hole_d =  2.60;  /* Hold down hole diameter (Spec Figure 2-3) */
hole_o =  2.90;  /* Hold down hole offset (Spec Figure 2-4): 50.95 - 48.05 */
pin_d  =  1.50;  /* Hold down pin diameter (ISO 68-1 M2 screw OD) minus
                  * tolerance */
koa_w  =  5.80;  /* Component and routing keep out area for hold down solution
                  * (Spec Figure 2-8) */
body_t =  2.00;  /* Bracket body thickness (Spec Figure 2-6): 1.00 + 2.4 MAX */
card_t =  1.00;  /* Card thickness (Spec Section 2.2.3 and Figure 2-6) */
ext_w  =  0.75;  /* Extrusion width (used for supports) */
lyr_h  =  0.25;  /* Layer height (used for supports) */
drp_t  =  0.25;  /* Bridge droop tolerance (used for Half-Mini component and
                  * routing keep out areas) */
top_t  =  0.25;  /* Top layer tolerance (used for card thickness in Full-Mini
                  * component and routing keep out areas) */
$fn = 32;

/* Full-Mini-end component and routing keep out areas.  Should be used in
 * difference(). */
module _pcie_mc_pin_bracket_fm_koas()
{
	union() {
		for (koa_x = [hole_s / -2 - koa_w * 1.5,
				hole_s / 2 - koa_w * 0.5]) {
			translate(v = [koa_x, koa_w / -2, card_t])
				cube(size = [koa_w * 2, koa_w * 2,
					(body_t - card_t) * 2]);
		}
	}
}

/* Full-Mini-end hold down holes.  Should be used in difference(). */
module _pcie_mc_pin_bracket_fm_holes()
{
	union() {
		for (hole_x = [hole_s / -2, hole_s / 2]) {
			translate(v = [hole_x, 0, card_t / -2])
				cylinder(h = card_t * 2, r = hole_d / 2);
		}
	}
}

/* Half-Mini-end component and routing keep out areas.  Should be used in
 * difference(). */
module _pcie_mc_pin_bracket_hm_koas()
{
	union() {
		translate(v = [card_w * -1, koa_w * -1.5, -1 * card_t])
			cube(size = [card_w * 2, koa_w * 2, card_t * 2]);
	}
}

/* Half-Mini-end hold down pins. */
module _pcie_mc_pin_bracket_hm_pins()
{
	union() {
		for (pin_x = [hole_s / -2, hole_s / 2]) {
			translate(v = [pin_x, 0, 0])
				cylinder(h = card_t, r = pin_d / 2);
		}
	}
}

/* Half-Mini-end hold down supports. */
module _pcie_mc_pin_bracket_hm_supports()
{
	union() {
		for (sup_x = [hole_s / -2, hole_s / 2]) {
			translate(v = [sup_x, koa_w / -2 + ext_w * 0.5,
				lyr_h / 4]) cube(size = [koa_w, ext_w * 1.0,
					lyr_h / 2], center = true);
			for (x = [koa_w / -2 + ext_w, koa_w / 2 - ext_w]) {
				translate(v = [sup_x + x, koa_w / -2 + ext_w,
					0]) cylinder(h = card_t, r = ext_w);
			}
		}
	}
}

/* Bracket body. */
module _pcie_mc_pin_bracket_body()
{
	translate(v = [0, 0, body_t / 2])
		cube(size = [card_w, span_l + koa_w, body_t], center = true);
}

/* Bracket void. */
module _pcie_mc_pin_bracket_void()
{
	hull() {
		for (cyl_x = [-1 * (card_w / 2 - koa_w - brdg_f),
				1 * (card_w / 2 - koa_w - brdg_f)]) {
			translate(v = [cyl_x,
				(span_l + koa_w) / 2 - brdg_w - brdg_f,
				body_t / -2])
					cylinder(r = brdg_f, h = body_t * 2);
		}
		translate(v = [card_w / -2 + koa_w, (span_l + koa_w) / -2 - 1,
			body_t / -2]) cube(size = [card_w - koa_w * 2, 1,
				body_t * 2]);
	}
}

/* Full bracket. */
module pcie_mc_pin_bracket()
{
	union() {
		difference() {
			_pcie_mc_pin_bracket_body();
			_pcie_mc_pin_bracket_void();
			translate(v = [0, (span_l + koa_w) / 2 - hole_o, 0])
				_pcie_mc_pin_bracket_fm_koas();
			translate(v = [0, (span_l + koa_w) / 2 - hole_o, 0])
				_pcie_mc_pin_bracket_fm_holes();
			translate(v = [0, (span_l + koa_w) / -2 + hole_o, 0])
				_pcie_mc_pin_bracket_hm_koas();
		}
		translate(v = [0, (span_l + koa_w) / -2 + hole_o, 0])
			_pcie_mc_pin_bracket_hm_pins();
		translate(v = [0, (span_l) / -2, 0])
			_pcie_mc_pin_bracket_hm_supports();
	}
}

function pcie_mc_pin_bracket_width() = card_w;
function pcie_mc_pin_bracket_length() = span_l + koa_w;
function pcie_mc_pin_bracket_thickness() = body_t;
function pcie_mc_pin_bracket_void_length() = card_w - koa_w * 2;
function pcie_mc_pin_bracket_void_width() = span_l + koa_w - brdg_w;
function pcie_mc_pin_bracket_bridge_fillet() = brdg_f;
